/*
 * res_conv.c - Asterisk module
 * 
 * redice li <redice_li@yahoo.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "asterisk.h"
ASTERISK_FILE_VERSION(__FILE__, "$Revision: $")

#include "asterisk/lock.h"
#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/translate.h"
#include "asterisk/options.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"
#include "asterisk/cli.h"
#include "asterisk/frame.h"

static int usecnt =0;

static int split_ext(char *filename, char **name, char **ext)
{
	char *p;
	*name = filename;
	p = strrchr(filename, '.');
	if (p){
		*p='\0';
		*ext = p+1;
		return 0;
	}
	return -1;
}

static int cli_audio_convert(int fd, int argc, char *argv[])
{
	int ret=RESULT_FAILURE;
	struct ast_filestream *fs_in=NULL, *fs_out=NULL;
	struct ast_frame *f;
	struct timeval start;
	int    cost;
	char   *file_in=NULL, *file_out=NULL;
	char   *name, *ext;
	
	if (argc!=3) 
		return RESULT_SHOWUSAGE;

	usecnt++;
	file_in = ast_strdupa(argv[1]);
	file_out = ast_strdupa(argv[2]);
	if ((!file_in) || (!file_out)){
		goto fail_out;
	}
	
	if (split_ext(file_in, &name, &ext))
		goto fail_out;

	fs_in = ast_readfile(name, ext, NULL, O_RDONLY, 0, 0);
	if (!fs_in){
		ast_cli(fd, "Unable open input file: %s\n", argv[1]);
		goto fail_out;
	}
	
	if (split_ext(file_out, &name, &ext)) 
		goto fail_out;

	fs_out = ast_writefile(name, ext, NULL, O_CREAT|O_TRUNC|O_WRONLY, 0, 0644);
	if (!fs_out){
		ast_cli(fd, "Unable to open output stream\n");
		goto fail_out;
	}

	start = ast_tvnow();

	while((f=ast_readframe(fs_in))!=NULL)
		ast_writestream(fs_out, f);

	cost = ast_tvdiff_ms(ast_tvnow(), start);

	ast_cli(fd, "Convert from %s to %s in %dms\n", file_in, file_out, cost);
	ret=0;

fail_out:
	if (fs_out) ast_closestream(fs_out);
	if (fs_in) ast_closestream(fs_in);
	usecnt--;
	return ret;
}

static char usage_audio_convert[] ="Usage: convert <file_in> <file_out>\n";
static struct ast_cli_entry audio_convert_cli={
	{"convert",NULL}, cli_audio_convert, "Convert audio files", usage_audio_convert
};

int unload_module(void)
{
	return ast_cli_unregister(&audio_convert_cli);
}

int load_module(void)
{
	return ast_cli_register(&audio_convert_cli);
}

char *description(void)
{
	return "Sample audio conversion module by redice";
}

int usecount(void)
{
	return usecnt;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
